describe("UrlEngine", function(){
	var $scope;
	var location;
	var urlEngine;

	var zhPromise = new ZHPromise(Q);
	var _eventAggregator;
	var screenService;
	var project = {};

	beforeEach(function(){
		window.templateUrl = "../../dyman/templates/dyman/";
		window.serviceUrl  = "../backend/service/index.php";
		module("ZeedhiApp");
		module("ZeedhiServices");

		project = {
			shouldBackToMainWindowOnReload: false,
			shouldBackToMainWindowOnReloadExceptions: [],
			mainWindow: "projectMainWindow"
		};

		inject(function($location, $rootScope, eventAggregator){
			location = $location;
			$scope = $rootScope;
			screenService = {
				openWindow : zhPromise.when
			};
			_eventAggregator = eventAggregator;
			urlEngine = new UrlEngine(location, $scope, _eventAggregator, screenService);
		});
	});

	it("UrlEngine.init should open the mainWindow when URL hasn't a containerName", function(done){
		spyOn(screenService, "openWindow").and.callFake(function(containerName){
			expect(containerName).toBe(project.mainWindow);
			done();
			return zhPromise.when();
		});
		urlEngine.init(project);
	});

	it("UrlEngine.init should open the container passed by URL - containerName without hash", function(done){
		location.path("myContainerName");
		spyOn(screenService, "openWindow").and.callFake(function(containerName){
			expect(containerName).toBe("myContainerName");
			done();
			return zhPromise.when();
		});
		urlEngine.init(project);
	});

	it("UrlEngine.init should open the container passed by URL - containerName with hash", function(done){
		location.path("myPackage");
		location.hash("myContainerName");
		spyOn(screenService, "openWindow").and.callFake(function(containerName){
			expect(containerName).toBe("myPackage#myContainerName");
			done();
			return zhPromise.when();
		});
		urlEngine.init(project);
	});

	it("UrlEngine.init should open the mainWindow when shouldBackToMainWindowOnReload = true - container without hash passed by URL", function(done){
		project.shouldBackToMainWindowOnReload = true;
		location.path("myPackage");
		spyOn(screenService, "openWindow").and.callFake(function(containerName){
			expect(containerName).toBe(project.mainWindow);
			done();
			return zhPromise.when();
		});
		urlEngine.init(project);
	});

	it("UrlEngine.init should open the mainWindow when shouldBackToMainWindowOnReload = true - container with hash passed by URL", function(done){
		project.shouldBackToMainWindowOnReload = true;
		location.path("myPackage");
		location.hash("myContainerName");
		spyOn(screenService, "openWindow").and.callFake(function(containerName){
			expect(containerName).toBe(project.mainWindow);
			done();
			return zhPromise.when();
		});
		urlEngine.init(project);
	});

	it("UrlEngine.init should back to the mainWindow when container is an exception and the default rule isn't back to mainWindow", function(done){
		project.shouldBackToMainWindowOnReload = false;
		project.shouldBackToMainWindowOnReloadExceptions = ["myContainerName"];
		location.path("myContainerName");
		spyOn(screenService, "openWindow").and.callFake(function(containerName){
			expect(containerName).toBe(project.mainWindow);
			done();
			return zhPromise.when();
		});
		urlEngine.init(project);
	});

	it("UrlEngine.init shouldn't back to the mainWindow when container is an exception and the default rule is back to mainWindow", function(done){
		project.shouldBackToMainWindowOnReload = true;
		project.shouldBackToMainWindowOnReloadExceptions = ["myContainerName"];
		location.path("myContainerName");
		spyOn(screenService, "openWindow").and.callFake(function(containerName){
			expect(containerName).toBe("myContainerName");
			done();
			return zhPromise.when();
		});
		urlEngine.init(project);
	});

	it("UrlEngine.init shouldn't back to the mainWindow when container passed by URL and 'inDevToolsMode'", function(done){
		location.path("myContainerName");
		location.search("devtools", true);
		spyOn(screenService, "openWindow").and.callFake(function(containerName){
			expect(containerName).toBe("myContainerName");
			done();
			return zhPromise.when();
		});
		urlEngine.init(project);
	});

	it("UrlEngine.init should add containerName in $location.url when ON_CONTAINER_OPEN is published and changedByUrl - initial URL without containerName", function(){
		var container = {
			fileName: "myContainer.json"
		};
		urlEngine.init(project);
		_eventAggregator.publish(EventAggregator.ON_CONTAINER_OPEN, {container: container});
		expect(location.url()).toBe("/myContainer");
	});

	it("UrlEngine.init should add containerName in $location.url when ON_CONTAINER_OPEN is published and changedByUrl - initial URL with containerName", function(){
		var container = {
			fileName: "myContainer.json"
		};
		location.url(location.url() + "/anotherContainerName");
		urlEngine.init(project);
		_eventAggregator.publish(EventAggregator.ON_CONTAINER_OPEN, {container: container});
		expect(location.url()).toBe("/myContainer");
	});

	it("UrlEngine.init shouldn't change $location.url when ON_CONTAINER_OPEN is published and isn't changedByUrl", function(){
		var container = {
			fileName: "myContainer.json"
		};
		var oldLocationUrl = location.url();
		urlEngine.init(project);
		$scope.$broadcast("$locationChangeSuccess");
		_eventAggregator.publish(EventAggregator.ON_CONTAINER_OPEN, {container: container});
		expect(location.url()).toBe(oldLocationUrl);
	});

	it("UrlEngine.init should add query to $location.url when ON_CONTAINER_OPEN is published, changedByUrl, and changedByInit", function(){
		var container = {
			fileName: "myContainer.json"
		};
		location.url(location.url() + "?myQuery=true");
		urlEngine.init(project);
		_eventAggregator.publish(EventAggregator.ON_CONTAINER_OPEN, {container: container});
		expect(location.url()).toBe("/myContainer?myQuery=true");
	});

	it("UrlEngine.init should publish locationChangeStart event", function(done){
		spyOn(_eventAggregator, "publish").and.callFake(function(eventName, eventParameters){
			expect(eventName).toBe("locationChangeStart");
			done();
		});
		urlEngine.init(project);
		$scope.$broadcast("$locationChangeStart");
	});

	it("UrlEngine.init should publish locationChangeSuccess event", function(done){
		spyOn(_eventAggregator, "publish").and.callFake(function(eventName, eventParameters){
			expect(eventName).toBe("locationChangeSuccess");
			done();
		});
		urlEngine.init(project);
		$scope.$broadcast("$locationChangeSuccess");
	});

	it("UrlEngine.init should open the container passed by URL in locationChangeSuccess event when isn't changedByOpen", function(done){
		urlEngine.init(project);
		location.path("myContainerName");
		spyOn(screenService, "openWindow").and.callFake(function(containerName){
			expect(containerName).toBe("myContainerName");
			done();
			return zhPromise.when();
		});
		$scope.$broadcast("$locationChangeSuccess");
	});

	it("UrlEngine.init shouldn't open the container passed by URL in locationChangeSuccess event when is changedByOpen", function(){
		var container = {
			fileName: "myContainer.json"
		};
		urlEngine.init(project);
		spyOn(screenService, "openWindow");
		_eventAggregator.publish(EventAggregator.ON_CONTAINER_OPEN, {container: container});
		$scope.$broadcast("$locationChangeSuccess");
		expect(screenService.openWindow).not.toHaveBeenCalled();
	});
});